@@ -22,22 +22,23 @@ import numpy
 
 def gate_strain_for_output_frames(pipeline, head, control):
 	controltee = pipeparts.mktee(pipeline, control)
-	head = pipeparts.mkgate(pipeline, mkqueue(pipeline, head), control = mkqueue(pipeline, controltee), threshold = 0, leaky = True)
+	control = mkaudiorate(pipeline, mkqueue(pipeline, controltee))
+	head = pipeparts.mkgate(pipeline, mkqueue(pipeline, head), control = control, threshold = 0, leaky = True)
 	control = mkqueue(pipeline, controltee)
 	return head, control
 
 def gate_other_with_strain(pipeline, other, strain):
-	other = pipeparts.mkgate(pipeline, other, control = mkqueue(pipeline, strain), threshold=0, leaky = True)
+	other = pipeparts.mkgate(pipeline, mkqueue(pipeline, other), control = mkqueue(pipeline, strain), threshold=0, leaky = True)
 	other = mkaudiorate(pipeline, other)
-	other = mkreblock(pipeline, other)
 	return other
 
 def mkqueue(pipeline, head):
 	return pipeparts.mkqueue(pipeline, head, max_size_time = 0, max_size_buffers = 0, max_size_bytes = 0)
 	
 def mkaudiorate(pipeline, head):
-	#return pipeparts.mkqueue(pipeline, head)
-	return pipeparts.mkaudiorate(pipeline, head, skip_to_first = True, silent = False)
+	head = pipeparts.mkaudiorate(pipeline, head, skip_to_first = True, silent = False)
+	head = mkreblock(pipeline, head)
+	return head
 
 def mkreblock(pipeline, head):
 	return pipeparts.mkreblock(pipeline, head, block_duration = gst.SECOND)
@@ -64,7 +65,6 @@ def caps_and_progress_and_resample(pipeline, head, caps, progress_name, new_caps
 	head = pipeparts.mkresample(pipeline, head, quality = 9)
 	head = pipeparts.mkcapsfilter(pipeline, head, new_caps)
 	head = mkaudiorate(pipeline, head)
-	head = mkreblock(pipeline, head)
 	return head
 
 def caps_and_progress_and_upsample(pipeline, head, caps, progress_name, new_caps):
@@ -73,6 +73,7 @@ def caps_and_progress_and_upsample(pipeline, head, caps, progress_name, new_caps
 	head = pipeparts.mkprogressreport(pipeline, head, "progress_src_%s" % progress_name)
 	head = pipeparts.mkgeneric(pipeline, head, "lal_constant_upsample")
 	head = pipeparts.mkcapsfilter(pipeline, head, new_caps)
+	head = mkaudiorate(pipeline, head)
 	head = pipeparts.mktee(pipeline, head)
 	return head
 
@@ -80,7 +81,12 @@ def resample(pipeline, head, caps):
 	head = pipeparts.mkresample(pipeline, head, quality = 9)
 	head = pipeparts.mkcapsfilter(pipeline, head, caps)
 	#head = mkaudiorate(pipeline, head)
-	head = mkreblock(pipeline, head)
+	return head
+
+def undersample(pipeline, head, caps):
+	head = pipeparts.mkaudioundersample(pipeline, head)
+	head = pipeparts.mkcapsfilter(pipeline, head, caps)
+	#head = mkaudiorate(pipeline, head)
 	return head
 
 def mkmultiplier(pipeline, srcs, caps, sync = True):
@@ -112,13 +118,11 @@ def list_srcs(pipeline, *args):
 		out.append(mkqueue(pipeline, src))
 	return tuple(out)
 
-def average_calib_factors(pipeline, head, var, expected, N, caps, td, hold_time):
+def average_calib_factors(pipeline, head, var, expected, N, caps, hold_time, Nav):
 	head = pipeparts.mkaudioconvert(pipeline, head)
 	head = pipeparts.mkcapsfilter(pipeline, head, caps)
-	head = pipeparts.mkgeneric(pipeline, head, "lal_check_calib_factors", variance = var, default = expected, wait_time_to_new_expected = hold_time)
-	head = mkaudiorate(pipeline, head)
-	head = pipeparts.mkfirbank(pipeline, head, fir_matrix = [numpy.ones(N)/N], time_domain = td)
-	head = resample(pipeline, head, caps)
+	head = pipeparts.mkgeneric(pipeline, head, "lal_check_calib_factors", variance = var, default = expected, wait_time_to_new_expected = hold_time, median_array_size = N)
+	head = pipeparts.mkfirbank(pipeline, head, fir_matrix = [numpy.ones(Nav)/Nav])
 	return head
 
 def merge_into_complex(pipeline, real, imag, real_caps, complex_caps):
@@ -126,7 +130,6 @@ def merge_into_complex(pipeline, real, imag, real_caps, complex_caps):
 	head = pipeparts.mkaudioconvert(pipeline, head)
 	head = pipeparts.mkcapsfilter(pipeline, head, "audio/x-raw-float")
 	head = mkaudiorate(pipeline, head) # This audiorate is necessary! Probaly once lal_interleave gets fixed it won't be
-	head = mkreblock(pipeline, head)
 	head = pipeparts.mktogglecomplex(pipeline,head)
 	head = pipeparts.mkcapsfilter(pipeline, head, complex_caps)
 	return head
